@@ -8,6 +8,7 @@
 #include <qcamera.h>
 #include <qmediacapturesession.h>
 #include <private/qplatformcamera_p.h>
+#include <private/qplatformscreencapture_p.h>
 #include <private/qplatformmediaintegration_p.h>
 #include <private/qplatformmediacapture_p.h>
 
@@ -58,6 +59,7 @@ QT_BEGIN_NAMESPACE
         id: captureSession
         camera: Camera {
             id: camera
+            active: true
         }
         audioInput: AudioInput {}
         recorder: MediaRecorder {
@@ -236,6 +238,8 @@ QMediaRecorder::RecorderState QMediaRecorder::recorderState() const
 }
 
 /*!
+    \property QMediaRecorder::error
+
     Returns the current error state.
 
     \sa errorString()
@@ -254,6 +258,8 @@ QMediaRecorder::Error QMediaRecorder::error() const
     \sa error
 */
 /*!
+    \property QMediaRecorder::errorString
+
     Returns a string describing the current error state.
 
     \sa error()
@@ -281,6 +287,11 @@ qint64 QMediaRecorder::duration() const
 {
     return d_func()->control ? d_func()->control->duration() : 0;
 }
+/*!
+    \fn void QMediaRecorder::encoderSettingsChanged()
+
+    Signals when the encoder settings change.
+*/
 /*!
     \qmlmethod QtMultimedia::MediaRecorder::record()
     \brief Starts recording.
@@ -327,9 +338,10 @@ void QMediaRecorder::record()
     } else {
         auto oldMediaFormat = d->encoderSettings.mediaFormat();
         auto camera = d->captureSession->camera();
-        auto flags = camera && camera->isActive() ? QMediaFormat::RequiresVideo
-                                                  : QMediaFormat::NoFlags;
-        d->encoderSettings.resolveFormat(flags);
+        auto screenCapture = d->captureSession->screenCapture();
+        bool hasVideo = (camera && camera->isActive()) || (screenCapture && screenCapture->isActive());
+
+        d->encoderSettings.resolveFormat(hasVideo ? QMediaFormat::RequiresVideo : QMediaFormat::NoFlags);
         d->control->clearActualLocation();
         d->control->clearError();
 
@@ -496,6 +508,8 @@ void QMediaRecorder::stop()
 */
 
 /*!
+    \property QMediaRecorder::metaData
+
     Returns the metaData associated with the recording.
 */
 QMediaMetaData QMediaRecorder::metaData() const
@@ -519,6 +533,9 @@ void QMediaRecorder::setMetaData(const QMediaMetaData &metaData)
         d->control->setMetaData(metaData);
 }
 
+/*!
+    Adds \a metaData to the recorded media.
+*/
 void QMediaRecorder::addMetaData(const QMediaMetaData &metaData)
 {
     auto data = this->metaData();
@@ -544,6 +561,9 @@ void QMediaRecorder::addMetaData(const QMediaMetaData &metaData)
     once.
 */
 
+/*!
+    Returns the media capture session.
+*/
 QMediaCaptureSession *QMediaRecorder::captureSession() const
 {
     Q_D(const QMediaRecorder);
@@ -592,7 +612,11 @@ QMediaCaptureSession *QMediaRecorder::captureSession() const
 
     \brief This property holds the current MediaFormat of the recorder.
 */
+/*!
+    \property QMediaRecorder::mediaFormat
 
+    Returns the recording media format.
+*/
 QMediaFormat QMediaRecorder::mediaFormat() const
 {
     Q_D(const QMediaRecorder);
@@ -619,6 +643,11 @@ QMediaRecorder::EncodingMode QMediaRecorder::encodingMode() const
     return d->encoderSettings.encodingMode();
 }
 
+/*!
+    \fn void QMediaRecorder::encodingModeChanged()
+
+    Signals when the encoding mode changes.
+*/
 /*!
     Sets the encoding \a mode setting.
 
@@ -637,12 +666,22 @@ void QMediaRecorder::setEncodingMode(EncodingMode mode)
     emit encodingModeChanged();
 }
 
+/*!
+    \property QMediaRecorder::quality
+
+    Returns the recording quality.
+*/
 QMediaRecorder::Quality QMediaRecorder::quality() const
 {
     Q_D(const QMediaRecorder);
     return d->encoderSettings.quality();
 }
 
+/*!
+    \fn void QMediaRecorder::qualityChanged()
+
+    Signals when the recording quality changes.
+*/
 void QMediaRecorder::setQuality(Quality quality)
 {
     Q_D(QMediaRecorder);
@@ -662,6 +701,11 @@ QSize QMediaRecorder::videoResolution() const
     return d->encoderSettings.videoResolution();
 }
 
+/*!
+    \fn void QMediaRecorder::videoResolutionChanged()
+
+    Signals when the video recording resolution changes.
+*/
 /*!
     Sets the resolution of the encoded video to \a{size}.
 
@@ -693,6 +737,11 @@ qreal QMediaRecorder::videoFrameRate() const
     return d->encoderSettings.videoFrameRate();
 }
 
+/*!
+    \fn void QMediaRecorder::videoFrameRateChanged()
+
+    Signals when the recording video frame rate changes.
+*/
 /*!
     Sets the video \a frameRate.
 
@@ -717,6 +766,11 @@ int QMediaRecorder::videoBitRate() const
     return d->encoderSettings.videoBitRate();
 }
 
+/*!
+    \fn void QMediaRecorder::videoBitRateChanged()
+
+    Signals when the recording video bit rate changes.
+*/
 /*!
     Sets the video \a bitRate in bits per second.
 */
@@ -738,6 +792,11 @@ int QMediaRecorder::audioBitRate() const
     return d->encoderSettings.audioBitRate();
 }
 
+/*!
+    \fn void QMediaRecorder::audioBitRateChanged()
+
+    Signals when the recording audio bit rate changes.
+*/
 /*!
     Sets the audio \a bitRate in bits per second.
 */
@@ -759,6 +818,11 @@ int QMediaRecorder::audioChannelCount() const
     return d->encoderSettings.audioChannelCount();
 }
 
+/*!
+    \fn void QMediaRecorder::audioChannelCountChanged()
+
+    Signals when the recording audio channel count changes.
+*/
 /*!
     Sets the number of audio \a channels.
 
@@ -782,7 +846,11 @@ int QMediaRecorder::audioSampleRate() const
     Q_D(const QMediaRecorder);
     return d->encoderSettings.audioSampleRate();
 }
+/*!
+    \fn void QMediaRecorder::audioSampleRateChanged()
 
+    Signals when the recording audio sample rate changes.
+*/
 /*!
     Sets the audio \a sampleRate in Hz.
 
