@@ -11,6 +11,7 @@
 #include <qaudiobuffer.h>
 #include "qffmpegaudioinput_p.h"
 #include <private/qplatformcamera_p.h>
+#include <private/qplatformvideosource_p.h>
 #include "qffmpegvideobuffer_p.h"
 #include "qffmpegmediametadata_p.h"
 #include "qffmpegencoderoptions_p.h"
@@ -24,7 +25,7 @@ extern "C" {
 
 QT_BEGIN_NAMESPACE
 
-Q_LOGGING_CATEGORY(qLcFFmpegEncoder, "qt.multimedia.ffmpeg.encoder")
+static Q_LOGGING_CATEGORY(qLcFFmpegEncoder, "qt.multimedia.ffmpeg.encoder")
 
 namespace QFFmpeg
 {
@@ -41,8 +42,8 @@ Encoder::Encoder(const QMediaEncoderSettings &settings, const QUrl &url)
     formatContext->url = (char *)av_malloc(encoded.size() + 1);
     memcpy(formatContext->url, encoded.constData(), encoded.size() + 1);
     formatContext->pb = nullptr;
-    avio_open2(&formatContext->pb, formatContext->url, AVIO_FLAG_WRITE, nullptr, nullptr);
-    qCDebug(qLcFFmpegEncoder) << "opened" << formatContext->url;
+    auto result = avio_open2(&formatContext->pb, formatContext->url, AVIO_FLAG_WRITE, nullptr, nullptr);
+    qCDebug(qLcFFmpegEncoder) << "opened" << result << formatContext->url;
 
     muxer = new Muxer(this);
 }
@@ -58,10 +59,33 @@ void Encoder::addAudioInput(QFFmpegAudioInput *input)
     input->setRunning(true);
 }
 
-void Encoder::addVideoSource(QPlatformCamera *source)
+void Encoder::addVideoSource(QPlatformVideoSource * source)
 {
-    videoEncode = new VideoEncoder(this, source, settings);
-    connect(source, &QPlatformCamera::newVideoFrame, this, &Encoder::newVideoFrame);
+    auto frameFormat = source->frameFormat();
+
+    if (!frameFormat.isValid()) {
+        qCWarning(qLcFFmpegEncoder) << "Cannot add source; invalid vide frame format";
+        return;
+    }
+
+    std::optional<AVPixelFormat> hwPixelFormat = source->ffmpegHWPixelFormat()
+            ? AVPixelFormat(*source->ffmpegHWPixelFormat())
+            : std::optional<AVPixelFormat>{};
+
+    qCDebug(qLcFFmpegEncoder) << "adding video source" << source->metaObject()->className() << ":"
+                              << "pixelFormat=" << frameFormat.pixelFormat()
+                              << "frameSize=" << frameFormat.frameSize()
+                              << "frameRate=" << frameFormat.frameRate() << "ffmpegHWPixelFormat="
+                              << (hwPixelFormat ? *hwPixelFormat : AV_PIX_FMT_NONE);
+
+    auto veUPtr = std::make_unique<VideoEncoder>(this, settings, frameFormat, hwPixelFormat);
+    if (veUPtr->isValid()) {
+        auto ve = veUPtr.release();
+        auto conn = connect(source, &QPlatformVideoSource::newVideoFrame,
+                            [=](const QVideoFrame &frame) { ve->addFrame(frame); });
+        videoEncoders.append(ve);
+        connections.append(conn);
+    }
 }
 
 void Encoder::start()
@@ -71,23 +95,34 @@ void Encoder::start()
     formatContext->metadata = QFFmpegMetaData::toAVMetaData(metaData);
 
     int res = avformat_write_header(formatContext, nullptr);
-    if (res < 0)
-        qWarning() << "could not write header" << res;
+    if (res < 0) {
+        qWarning() << "could not write header, error:" << res << err2str(res);
+        emit error(QMediaRecorder::ResourceError, "Cannot start writing the stream");
+        return;
+    }
+
+    qCDebug(qLcFFmpegEncoder) << "stream header is successfully written";
 
     muxer->start();
     if (audioEncode)
         audioEncode->start();
-    if (videoEncode)
-        videoEncode->start();
+    for (auto *videoEncoder : videoEncoders)
+        if (videoEncoder->isValid())
+            videoEncoder->start();
+
     isRecording = true;
 }
 
+EncodingFinalizer::EncodingFinalizer(Encoder *e) : encoder(e) {
+    connect(this, &QThread::finished, this, &QObject::deleteLater);
+}
+
 void EncodingFinalizer::run()
 {
     if (encoder->audioEncode)
         encoder->audioEncode->kill();
-    if (encoder->videoEncode)
-        encoder->videoEncode->kill();
+    for (auto &videoEncoder : encoder->videoEncoders)
+        videoEncoder->kill();
     encoder->muxer->kill();
 
     int res = av_write_trailer(encoder->formatContext);
@@ -95,17 +130,18 @@ void EncodingFinalizer::run()
         qWarning() << "could not write trailer" << res;
 
     avformat_free_context(encoder->formatContext);
-    qDebug() << "    done finalizing.";
+    qCDebug(qLcFFmpegEncoder) << "    done finalizing.";
     emit encoder->finalizationDone();
     delete encoder;
-    deleteLater();
 }
 
 void Encoder::finalize()
 {
-    qDebug() << ">>>>>>>>>>>>>>> finalize";
+    qCDebug(qLcFFmpegEncoder) << ">>>>>>>>>>>>>>> finalize";
+
+    for (auto &conn : connections)
+        disconnect(conn);
 
-    isRecording = false;
     auto *finalizer = new EncodingFinalizer(this);
     finalizer->start();
 }
@@ -114,8 +150,8 @@ void Encoder::setPaused(bool p)
 {
     if (audioEncode)
        audioEncode->setPaused(p);
-    if (videoEncode)
-       videoEncode->setPaused(p);
+    for (auto &videoEncoder : videoEncoders)
+       videoEncoder->setPaused(p);
 }
 
 void Encoder::setMetaData(const QMediaMetaData &metaData)
@@ -129,12 +165,6 @@ void Encoder::newAudioBuffer(const QAudioBuffer &buffer)
         audioEncode->addBuffer(buffer);
 }
 
-void Encoder::newVideoFrame(const QVideoFrame &frame)
-{
-    if (videoEncode && isRecording)
-        videoEncode->addFrame(frame);
-}
-
 void Encoder::newTimeStamp(qint64 time)
 {
     QMutexLocker locker(&timeMutex);
@@ -169,6 +199,7 @@ AVPacket *Muxer::takePacket()
 
 void Muxer::init()
 {
+    qCDebug(qLcFFmpegEncoder) << "Muxer::init started thread.";
 }
 
 void Muxer::cleanup()
@@ -184,7 +215,8 @@ bool QFFmpeg::Muxer::shouldWait() const
 void Muxer::loop()
 {
     auto *packet = takePacket();
-//    qCDebug(qLcFFmpegEncoder) << "writing packet to file" << packet->pts << packet->duration << packet->stream_index;
+    //   qCDebug(qLcFFmpegEncoder) << "writing packet to file" << packet->pts << packet->duration <<
+    //   packet->stream_index;
     av_interleaved_write_frame(encoder->formatContext, packet);
 }
 
@@ -221,6 +253,7 @@ static AVSampleFormat bestMatchingSampleFormat(AVSampleFormat requested, const A
 
 AudioEncoder::AudioEncoder(Encoder *encoder, QFFmpegAudioInput *input, const QMediaEncoderSettings &settings)
     : input(input)
+    , settings(settings)
 {
     this->encoder = encoder;
 
@@ -231,9 +264,17 @@ AudioEncoder::AudioEncoder(Encoder *encoder, QFFmpegAudioInput *input, const QMe
     auto codecID = QFFmpegMediaFormatInfo::codecIdForAudioCodec(settings.audioCodec());
     Q_ASSERT(avformat_query_codec(encoder->formatContext->oformat, codecID, FF_COMPLIANCE_NORMAL));
 
-    auto *avCodec = avcodec_find_encoder(codecID);
-
     AVSampleFormat requested = QFFmpegMediaFormatInfo::avSampleFormat(format.sampleFormat());
+
+    avCodec = QFFmpeg::findAVEncoder(codecID, {}, requested);
+
+    if (!avCodec)
+        avCodec = QFFmpeg::findAVEncoder(codecID);
+
+    qCDebug(qLcFFmpegEncoder) << "found audio codec" << avCodec->name;
+
+    Q_ASSERT(avCodec);
+
     AVSampleFormat bestSampleFormat = bestMatchingSampleFormat(requested, avCodec->sample_fmts);
 
     stream = avformat_new_stream(encoder->formatContext, nullptr);
@@ -251,14 +292,30 @@ AudioEncoder::AudioEncoder(Encoder *encoder, QFFmpegAudioInput *input, const QMe
     stream->codecpar->format = bestSampleFormat;
     stream->time_base = AVRational{ 1, format.sampleRate() };
 
-    Q_ASSERT(avCodec);
+    qCDebug(qLcFFmpegEncoder) << "set stream time_base" << stream->time_base.num << "/"
+                              << stream->time_base.den;
+}
+
+void AudioEncoder::open()
+{
+    AVSampleFormat requested = QFFmpegMediaFormatInfo::avSampleFormat(format.sampleFormat());
+
     codec = avcodec_alloc_context3(avCodec);
+
+    if (stream->time_base.num != 1 || stream->time_base.den != format.sampleRate()) {
+        qCDebug(qLcFFmpegEncoder) << "Most likely, av_format_write_header changed time base from"
+                                  << 1 << "/" << format.sampleRate() << "to"
+                                  << stream->time_base.num << "/" << stream->time_base.den;
+    }
+
+    codec->time_base = stream->time_base;
+
     avcodec_parameters_to_context(codec, stream->codecpar);
 
-    AVDictionary *opts = nullptr;
-    applyAudioEncoderOptions(settings, avCodec->name, codec, &opts);
+    AVDictionaryHolder opts;
+    applyAudioEncoderOptions(settings, avCodec->name, codec, opts);
 
-    int res = avcodec_open2(codec, avCodec, &opts);
+    int res = avcodec_open2(codec, avCodec, opts);
     qCDebug(qLcFFmpegEncoder) << "audio codec opened" << res;
     qCDebug(qLcFFmpegEncoder) << "audio codec params: fmt=" << codec->sample_fmt << "rate=" << codec->sample_rate;
 
@@ -305,6 +362,7 @@ QAudioBuffer AudioEncoder::takeBuffer()
 
 void AudioEncoder::init()
 {
+    open();
     if (input) {
         input->setFrameSize(codec->frame_size);
     }
@@ -343,7 +401,7 @@ void AudioEncoder::retrievePackets()
             break;
         }
 
-        //        qCDebug(qLcFFmpegEncoder) << "writing video packet" << packet->size << packet->pts << timeStamp(packet->pts, stream->time_base) << packet->stream_index;
+        // qCDebug(qLcFFmpegEncoder) << "writing audio packet" << packet->size << packet->pts << packet->dts;
         packet->stream_index = stream->id;
         encoder->muxer->addPacket(packet);
     }
@@ -358,7 +416,7 @@ void AudioEncoder::loop()
 //    qCDebug(qLcFFmpegEncoder) << "new audio buffer" << buffer.byteCount() << buffer.format() << buffer.frameCount() << codec->frame_size;
     retrievePackets();
 
-    AVFrame *frame = av_frame_alloc();
+    auto frame = makeAVFrame();
     frame->format = codec->sample_fmt;
 #if QT_FFMPEG_OLD_CHANNEL_LAYOUT
     frame->channel_layout = codec->channel_layout;
@@ -369,7 +427,7 @@ void AudioEncoder::loop()
     frame->sample_rate = codec->sample_rate;
     frame->nb_samples = buffer.frameCount();
     if (frame->nb_samples)
-        av_frame_get_buffer(frame, 0);
+        av_frame_get_buffer(frame.get(), 0);
 
     if (resampler) {
         const uint8_t *data = buffer.constData<uint8_t>();
@@ -378,14 +436,20 @@ void AudioEncoder::loop()
         memcpy(frame->buf[0]->data, buffer.constData<uint8_t>(), buffer.byteCount());
     }
 
-    frame->pts = samplesWritten;
+    const auto &timeBase = stream->time_base;
+    const auto pts = timeBase.den && timeBase.num
+            ? timeBase.den * samplesWritten / (codec->sample_rate * timeBase.num)
+            : samplesWritten;
+    setAVFrameTime(*frame, pts, timeBase);
     samplesWritten += buffer.frameCount();
 
     qint64 time = format.durationForFrames(samplesWritten);
     encoder->newTimeStamp(time/1000);
 
-//    qCDebug(qLcFFmpegEncoder) << "sending audio frame" << buffer.byteCount() << frame->pts << ((double)buffer.frameCount()/frame->sample_rate);
-    int ret = avcodec_send_frame(codec, frame);
+    //    qCDebug(qLcFFmpegEncoder) << "sending audio frame" << buffer.byteCount() << frame->pts <<
+    //    ((double)buffer.frameCount()/frame->sample_rate);
+
+    int ret = avcodec_send_frame(codec, frame.get());
     if (ret < 0) {
         char errStr[1024];
         av_strerror(ret, errStr, 1024);
@@ -393,24 +457,26 @@ void AudioEncoder::loop()
     }
 }
 
-VideoEncoder::VideoEncoder(Encoder *encoder, QPlatformCamera *camera, const QMediaEncoderSettings &settings)
-    : m_encoderSettings(settings)
-    , m_camera(camera)
+VideoEncoder::VideoEncoder(Encoder *encoder, const QMediaEncoderSettings &settings,
+                           const QVideoFrameFormat &format, std::optional<AVPixelFormat> hwFormat)
 {
     this->encoder = encoder;
 
     setObjectName(QLatin1String("VideoEncoder"));
-    qCDebug(qLcFFmpegEncoder) << "VideoEncoder" << settings.videoCodec();
-
-    auto format = m_camera->cameraFormat();
-    std::optional<AVPixelFormat> hwFormat = camera->ffmpegHWPixelFormat()
-            ? AVPixelFormat(*camera->ffmpegHWPixelFormat())
-            : std::optional<AVPixelFormat>{};
 
     AVPixelFormat swFormat = QFFmpegVideoBuffer::toAVPixelFormat(format.pixelFormat());
-    AVPixelFormat pixelFormat = hwFormat ? *hwFormat : swFormat;
-    frameEncoder = new VideoFrameEncoder(settings, format.resolution(), format.maxFrameRate(), pixelFormat, swFormat);
+    AVPixelFormat ffmpegPixelFormat =
+            hwFormat && *hwFormat != AV_PIX_FMT_NONE ? *hwFormat : swFormat;
+    auto frameRate = format.frameRate();
+    if (frameRate <= 0.) {
+        qWarning() << "Invalid frameRate" << frameRate << "; Using the default instead";
 
+        // set some default frame rate since ffmpeg has UB if it's 0.
+        frameRate = 30.;
+    }
+
+    frameEncoder = new VideoFrameEncoder(settings, format.frameSize(), frameRate, ffmpegPixelFormat,
+                                         swFormat);
     frameEncoder->initWithFormatContext(encoder->formatContext);
 }
 
@@ -422,18 +488,35 @@ VideoEncoder::~VideoEncoder()
 void VideoEncoder::addFrame(const QVideoFrame &frame)
 {
     QMutexLocker locker(&queueMutex);
-    if (!paused.loadRelaxed()) {
+
+    // Drop frames if encoder can not keep up with the video source data rate
+    const bool queueFull = videoFrameQueue.size() >= maxQueueSize;
+
+    if (queueFull) {
+        qCDebug(qLcFFmpegEncoder) << "Encoder frame queue full. Frame lost.";
+    } else if (!paused.loadRelaxed()) {
         videoFrameQueue.enqueue(frame);
+
+        locker.unlock(); // Avoid context switch on wake wake-up
+
         wake();
     }
 }
 
+bool VideoEncoder::isValid() const
+{
+    return !frameEncoder->isNull();
+}
+
 QVideoFrame VideoEncoder::takeFrame()
 {
     QMutexLocker locker(&queueMutex);
-    if (videoFrameQueue.isEmpty())
-        return QVideoFrame();
-    return videoFrameQueue.dequeue();
+
+    QVideoFrame frame;
+    if (!videoFrameQueue.isEmpty())
+        frame = videoFrameQueue.dequeue();
+
+    return frame;
 }
 
 void VideoEncoder::retrievePackets()
@@ -449,7 +532,7 @@ void VideoEncoder::init()
     qCDebug(qLcFFmpegEncoder) << "VideoEncoder::init started video device thread.";
     bool ok = frameEncoder->open();
     if (!ok)
-        encoder->error(QMediaRecorder::ResourceError, "Could not initialize encoder");
+        emit encoder->error(QMediaRecorder::ResourceError, "Could not initialize encoder");
 }
 
 void VideoEncoder::cleanup()
@@ -496,24 +579,23 @@ void VideoEncoder::loop()
 
 //    qCDebug(qLcFFmpegEncoder) << "new video buffer" << frame.startTime();
 
-    AVFrame *avFrame = nullptr;
+    AVFrameUPtr avFrame;
 
     auto *videoBuffer = dynamic_cast<QFFmpegVideoBuffer *>(frame.videoBuffer());
     if (videoBuffer) {
         // ffmpeg video buffer, let's use the native AVFrame stored in there
         auto *hwFrame = videoBuffer->getHWFrame();
         if (hwFrame && hwFrame->format == frameEncoder->sourceFormat())
-            avFrame = av_frame_clone(hwFrame);
+            avFrame.reset(av_frame_clone(hwFrame));
     }
 
     if (!avFrame) {
         frame.map(QVideoFrame::ReadOnly);
         auto size = frame.size();
-        avFrame = av_frame_alloc();
+        avFrame = makeAVFrame();
         avFrame->format = frameEncoder->sourceFormat();
         avFrame->width = size.width();
         avFrame->height = size.height();
-        av_frame_get_buffer(avFrame, 0);
 
         for (int i = 0; i < 4; ++i) {
             avFrame->data[i] = const_cast<uint8_t *>(frame.bits(i));
@@ -533,19 +615,21 @@ void VideoEncoder::loop()
         avFrame->opaque_ref = av_buffer_create(nullptr, 0, freeQVideoFrame, new QVideoFrameHolder{frame, img}, 0);
     }
 
-    if (baseTime.loadAcquire() < 0) {
+    if (baseTime.loadAcquire() == std::numeric_limits<qint64>::min()) {
         baseTime.storeRelease(frame.startTime() - lastFrameTime);
-//        qCDebug(qLcFFmpegEncoder) << ">>>> adjusting base time to" << baseTime.loadAcquire() << frame.startTime() << lastFrameTime;
+        qCDebug(qLcFFmpegEncoder) << ">>>> adjusting base time to" << baseTime.loadAcquire()
+                                  << frame.startTime() << lastFrameTime;
     }
 
     qint64 time = frame.startTime() - baseTime.loadAcquire();
     lastFrameTime = frame.endTime() - baseTime.loadAcquire();
-    avFrame->pts = frameEncoder->getPts(time);
+
+    setAVFrameTime(*avFrame, frameEncoder->getPts(time), frameEncoder->getTimeBase());
 
     encoder->newTimeStamp(time/1000);
 
-//    qCDebug(qLcFFmpegEncoder) << ">>> sending frame" << avFrame->pts << time;
-    int ret = frameEncoder->sendFrame(avFrame);
+    qCDebug(qLcFFmpegEncoder) << ">>> sending frame" << avFrame->pts << time << lastFrameTime;
+    int ret = frameEncoder->sendFrame(std::move(avFrame));
     if (ret < 0) {
         qCDebug(qLcFFmpegEncoder) << "error sending frame" << ret << err2str(ret);
         encoder->error(QMediaRecorder::ResourceError, err2str(ret));
@@ -555,3 +639,5 @@ void VideoEncoder::loop()
 }
 
 QT_END_NAMESPACE
+
+#include "moc_qffmpegencoder_p.cpp"
